/* -*- C++ -*-
 * $Id: flcolor.cc,v 1.7 2006/12/21 15:31:18 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flcolor.h"

static VALUE rb_fltk_use_color_classes(VALUE self){
#ifdef USE_COLOR_CLASSES
  return Qtrue;
#else // #ifdef USE_COLOR_CLASSES 
  return Qfalse;
#endif // #ifdef USE_COLOR_CLASSES 
}

#ifdef USE_COLOR_CLASSES

static VALUE rb_mFLTKColor;
static VALUE rb_mFLTKColorConst;

// Instances of FLTK::Color::Index represent indexes into FLTK's color
// table.

static VALUE rb_cFLColorIndex;

typedef struct _ColorIndex {
  Fl_Color index;
  int operator==(struct _ColorIndex &other) {
    return index == other.index;
  }
} ColorIndex;

// hashing 
static long index_hash(ColorIndex &index) {
  return ((long)index.index);
}

// Instances of FLTK::Color::RGB represent FLTK rgb colors.

static VALUE rb_cFLColorRGB;

typedef struct _ColorRGB {
  uchar r, g, b;
  int operator==(struct _ColorRGB &other) {
    return
      r == other.r &&
      g == other.g &&
      b == other.b;
  }
} ColorRGB;

// hashing 
static long rgb_hash(ColorRGB &rgb) {
  return rgb.r ^ rgb.g ^ rgb.b;
}

// equality of ColorIndex and ColorRGB 

static void fl_get_color(ColorIndex &index, ColorRGB &rgb) {
  Fl::get_color(index.index,rgb.r,rgb.g,rgb.b);
}

static int operator==(ColorIndex &index, ColorRGB &rgb) {
  ColorRGB index_rgb;
  fl_get_color(index,index_rgb);
  return index_rgb == rgb;
}

static int operator==(ColorRGB &rgb, ColorIndex &index) {
  ColorRGB index_rgb;
  fl_get_color(index,index_rgb); 
  return index_rgb == rgb;
}

static int index_rgb_equal(ColorIndex &x, ColorIndex &y) {
    ColorRGB x_rgb, y_rgb;
    fl_get_color(x,x_rgb);
    fl_get_color(y,y_rgb);
    return x_rgb == y_rgb;
}

// equality of VALUEs

#define RETURN_TRUE_IF_EQUAL(x,x_class,y,y_class) \
  x_class *x##_data; \
  y_class *y##_data; \
  Data_Get_Struct(x,x_class,x##_data); \
  Data_Get_Struct(y,y_class,y##_data); \
  if (*x##_data == *y##_data) \
    return Qtrue;

#define RETURN_EQUALITY(x,x_class,y,y_class) \
  RETURN_TRUE_IF_EQUAL(x,x_class,y,y_class) \
  return Qfalse;

// Implementation of FLTK::Color::Index 

static void index_free(ColorIndex *index) {
  delete index;
}

static VALUE rb_FLColorIndex_allocate(VALUE _class) {
  ColorIndex *index = new ColorIndex;
  return Data_Wrap_Struct(_class,0,index_free,index);
}

static VALUE rb_FLColorIndex_initialize_copy(VALUE self, VALUE other) {
  ColorIndex *self_index;
  ColorIndex *other_index;
  Data_Get_Struct(self,  ColorIndex,  self_index);
  Data_Get_Struct(other, ColorIndex, other_index);
  *self_index = *other_index;
  return self;
}

static VALUE rb_FLColorIndex_initialize(VALUE self, VALUE index) {
  ColorIndex *self_index;
  Data_Get_Struct(self,ColorIndex,self_index);
  uint arg_index = NUM2UINT(index);
  if ( arg_index & 0xffffff00 ) {
    rb_raise(rb_eRangeError,
	     "0x%x is not a valid color index",
	     arg_index);
  }
  self_index->index = (Fl_Color)arg_index;
  return self; 
}

static VALUE rb_FLColorIndex_hash(VALUE self) {
  ColorIndex *index;
  Data_Get_Struct(self,ColorIndex,index);
  return LONG2FIX(index_hash(*index));
}

static VALUE rb_FLColorIndex_eql(VALUE self, VALUE other) {
  if (self == other)
    return Qtrue; 
  if (!rb_obj_is_kind_of(other,rb_cFLColorIndex))
    return Qfalse;
  RETURN_EQUALITY(self,ColorIndex,other,ColorIndex);
}

static VALUE rb_FLColorIndex_equal(VALUE self, VALUE other) {
  if (self == other)
    return Qtrue;
  if (rb_obj_is_kind_of(other,rb_cFLColorIndex)) {
    RETURN_TRUE_IF_EQUAL(self,ColorIndex,other,ColorIndex);
    if (index_rgb_equal(*self_data,*other_data))
      return Qtrue;
    return Qfalse; 
  }
  if (rb_obj_is_kind_of(other,rb_cFLColorRGB)) {
    RETURN_EQUALITY(self,ColorIndex,other,ColorRGB);
  }
  return Qfalse; 
}

static VALUE rb_FLColorIndex_index(VALUE self) {
  ColorIndex *index;
  Data_Get_Struct(self,ColorIndex,index);
  return UINT2NUM((uint)(index->index));
}

#define FN_COLORINDEX_FIELD(field) \
VALUE rb_FLColorIndex_##field(VALUE self) { \
  ColorIndex *index; \
  uchar r, g, b; \
  Data_Get_Struct(self,ColorIndex,index); \
  Fl::get_color(index->index,r,g,b); \
  return UCHAR2NUM(field); \
}

static FN_COLORINDEX_FIELD(r);
static FN_COLORINDEX_FIELD(g);
static FN_COLORINDEX_FIELD(b);

// Implementation of FLTK::Color::RGB 

static void rgb_free(ColorRGB *rgb) {
  delete rgb;
}

static VALUE rb_FLColorRGB_allocate(VALUE _class) {
  ColorRGB *rgb = new ColorRGB;
  return Data_Wrap_Struct(_class,0,rgb_free,rgb);
}

static VALUE rb_FLColorRGB_initialize_copy(VALUE self, VALUE other) {
  ColorRGB *self_rgb;
  ColorRGB *other_rgb;
  Data_Get_Struct(self,  ColorRGB,  self_rgb);
  Data_Get_Struct(other, ColorRGB, other_rgb);
  *self_rgb = *other_rgb;
  return self;
}

static VALUE rb_FLColorRGB_initialize(VALUE self, VALUE r, VALUE g, VALUE b) {
  ColorRGB *rgb;
  Data_Get_Struct(self,ColorRGB,rgb);
  rgb->r = NUM2UCHAR(r);
  rgb->g = NUM2UCHAR(g);
  rgb->b = NUM2UCHAR(b);
  return self; 
}

static VALUE rb_FLColorRGB_hash(VALUE self) {
  ColorRGB *rgb;
  Data_Get_Struct(self,ColorRGB,rgb);
  return LONG2FIX(rgb_hash(*rgb));
}

static VALUE rb_FLColorRGB_eql(VALUE self, VALUE other) {
  if (self == other)
    return Qtrue;
  if (!rb_obj_is_kind_of(other,rb_cFLColorRGB))
    return Qfalse;
  RETURN_EQUALITY(self,ColorRGB,other,ColorRGB);
}

static VALUE rb_FLColorRGB_equal(VALUE self, VALUE other) {
  if (self == other)
    return Qtrue;
  if (rb_obj_is_kind_of(other,rb_cFLColorRGB)) {
    RETURN_EQUALITY(self,ColorRGB,other,ColorRGB);
  }
  if (rb_obj_is_kind_of(other,rb_cFLColorIndex)) {
    RETURN_EQUALITY(self,ColorRGB,other,ColorIndex); 
  }
  return Qfalse;
}

#define FN_COLORRGB_FIELD(field) \
VALUE rb_FLColorRGB_##field(int argc, VALUE argv[], VALUE self) { \
  ColorRGB *rgb; \
  VALUE arg; \
  Data_Get_Struct(self,ColorRGB,rgb); \
  switch( rb_scan_args(argc, argv, "01", &arg) ){ \
  case 0: return UCHAR2NUM(rgb->field); \
  case 1: rgb->field = NUM2UCHAR(arg) ; return Qnil; \
  } \
}

static FN_COLORRGB_FIELD(r);
static FN_COLORRGB_FIELD(g);
static FN_COLORRGB_FIELD(b);

// Convert between FLTK Fl_Colors and Ruby/FLTK VALUEs.

VALUE rb_fltk_color_to_value(Fl_Color color){
  if ((uint)color & 0xffffff00) { // an RGB value 
    ColorRGB *rgb = new ColorRGB;
    Fl::get_color(color,rgb->r,rgb->g,rgb->b);
    return Data_Wrap_Struct(rb_cFLColorRGB,0,rgb_free,rgb);
  } else { // an Fl_Color index 
    ColorIndex *index = new ColorIndex;
    index->index = color;
    return Data_Wrap_Struct(rb_cFLColorIndex,0,index_free,index);
  }
}

Fl_Color rb_fltk_value_to_color(VALUE value) {
  if (NIL_P(value)) {
    return ((Fl_Color) 0); // nil is black! 
  } else if (rb_obj_is_kind_of(value,rb_cFLColorRGB)) { // an RGB value 
    ColorRGB *rgb;
    Data_Get_Struct(value,ColorRGB,rgb);
    return fl_rgb_color(rgb->r,rgb->g,rgb->b);
  } else if (rb_obj_is_kind_of(value,rb_cFLColorIndex)) { // an Fl_Color index 
    ColorIndex *index;
    Data_Get_Struct(value,ColorIndex,index);
    return index->index;
  } else if (rb_obj_is_kind_of(value,rb_cInteger)) {
    return (Fl_Color)(NUM2UINT(value));
  } else {
    rb_raise(rb_eTypeError,
	     "cannot coerce a %s to an Fl_Color",
	     rb_obj_classname(value));
    return ((Fl_Color) 0); // never reached 
  }
}

#endif // #ifdef USE_COLOR_CLASSES 

extern "C" void
Init_flcolor(){

  DEF_MODULE_FUNC_Q(use_color_classes,0);

#ifdef USE_COLOR_CLASSES

  // module FLTK::Color 
  rb_mFLTKColor =
    rb_define_module_under(rb_mFLTK, "Color");

  // class FLTK::Color::Index 
  rb_cFLColorIndex =
    rb_define_class_under(rb_mFLTKColor, "Index", rb_cObject);
  rb_define_alloc_func(rb_cFLColorIndex,rb_FLColorIndex_allocate);
  DEF_METHOD(ColorIndex,initialize_copy,1);
  DEF_METHOD(ColorIndex,initialize,1);
  DEF_EQUALITY_METHODS(ColorIndex);
  DEF_METHOD(ColorIndex,index,0);
  DEF_METHOD(ColorIndex,r,0);
  DEF_METHOD(ColorIndex,g,0);
  DEF_METHOD(ColorIndex,b,0);

  // class FLTK::Color::RGB 
  rb_cFLColorRGB =
    rb_define_class_under(rb_mFLTKColor, "RGB", rb_cObject);
  rb_define_alloc_func(rb_cFLColorRGB,rb_FLColorRGB_allocate); 
  DEF_METHOD(ColorRGB,initialize_copy,1);
  DEF_METHOD(ColorRGB,initialize,3);
  DEF_EQUALITY_METHODS(ColorRGB);
  DEF_METHOD_EQ(ColorRGB,r,-1);
  DEF_METHOD_EQ(ColorRGB,g,-1);
  DEF_METHOD_EQ(ColorRGB,b,-1);

  // module FLTK::Color::Const 
  rb_mFLTKColorConst =
    rb_define_module_under(rb_mFLTKColor, "Const");
  rb_include_module(rb_mFLTKConst,rb_mFLTKColorConst);
  rb_include_module(rb_mFLTK,rb_mFLTKColorConst);

#endif // #ifdef USE_COLOR_CLASSES 

#ifdef USE_COLOR_CLASSES
#define COLOR_CONST(c) \
rb_define_const(rb_mFLTKColorConst,#c,COLOR2VALUE(FL_##c)) 
#else // #ifdef USE_COLOR_CLASSES 
#define COLOR_CONST(c) \
DEF_MODULE_CONST(#c,COLOR2VALUE(FL_##c))
#endif // #ifdef USE_COLOR_CLASSES 

  COLOR_CONST(BLACK);
  COLOR_CONST(RED);
  COLOR_CONST(DARK_RED);
  COLOR_CONST(GREEN);
  COLOR_CONST(DARK_GREEN);
  COLOR_CONST(YELLOW);
  COLOR_CONST(DARK_YELLOW);
  COLOR_CONST(BLUE);
  COLOR_CONST(DARK_BLUE);
  COLOR_CONST(MAGENTA);
  COLOR_CONST(DARK_MAGENTA);
  COLOR_CONST(CYAN);
  COLOR_CONST(DARK_CYAN);
  COLOR_CONST(WHITE);
  COLOR_CONST(FOREGROUND_COLOR);
  COLOR_CONST(BACKGROUND_COLOR);
  COLOR_CONST(BACKGROUND2_COLOR);
  COLOR_CONST(INACTIVE_COLOR);
  COLOR_CONST(SELECTION_COLOR);
  COLOR_CONST(GRAY);
  COLOR_CONST(GRAY0);
  COLOR_CONST(DARK3);
  COLOR_CONST(DARK2);
  COLOR_CONST(DARK1);
  COLOR_CONST(LIGHT1);
  COLOR_CONST(LIGHT2);
  COLOR_CONST(LIGHT3);

};
