/* -*- C++ -*-
 * $Id: flfile.cc,v 1.20 2006/12/29 04:56:54 jeremy_henty Exp $
 */

#include "fltk.h"

#ifdef HAVE_FL_FL_FILE_CHOOSER_H

#include "flfile.h"
#include "flbutton.h"
#include "flenum.h"
#include "flcolor.h"

#define DEF_FILE_BROWSER_CONST_NUM(c) \
  rb_define_const(rb_cFLFile_Browser,#c,INT2NUM(Fl_File_Browser::c))
#define DEF_FILE_CHOOSER_CONST_NUM(c) \
  rb_define_const(rb_cFLFile_Chooser,#c,INT2NUM(Fl_File_Chooser::c))

VALUE rb_cFLFile_Browser;
VALUE rb_cFLFile_Chooser;

static FN_S_NEW(File_Browser);
WIDGET_FNS(File_Browser);
static FN_GET_INT_SET_STR(File_Browser,load);
static FN_GETSET_INT(File_Browser,filetype);
static FN_GETSET_STR(File_Browser,filter);
static FN_GETSET_INT(File_Browser,iconsize);


static FN_CALL(File_Chooser,show);
static FN_CALL(File_Chooser,hide);
static FN_GETSET_STR(File_Chooser,filter);
static FN_GETSET_INT(File_Chooser,textfont);
static FN_GETSET_INT(File_Chooser,textsize);
static FN_GETSET_INT(File_Chooser,type);
static FN_GET_BOOL(File_Chooser,shown);
#ifdef HAVE_FL_FILE_CHOOSER_FILTER_VALUE
static FN_GETSET_INT(File_Chooser,filter_value);
#endif
static FN_GETSET_COLOR(File_Chooser,color);
static FN_GET_INT(File_Chooser,count);
static FN_GETSET_INT(File_Chooser,iconsize);
static FN_GETSET_STR_COPY(File_Chooser,label);
static FN_CALL(File_Chooser,rescan);
static FN_GET_BOOL(File_Chooser,visible);

static VALUE rb_FLFile_Chooser_new(VALUE klass, RBFLFile_Chooser *chooser)
{
  VALUE obj = Data_Wrap_Struct(klass,0,0,chooser);
  rb_fltk_objmap_aset((Fl_Widget*)chooser, obj);
  return obj;
};

static VALUE rb_FLFile_Chooser_s_new(int argc, VALUE argv[], VALUE klass)
{
  RBFLFile_Chooser *chooser;
  VALUE d,p,t,title;
  VALUE obj;

  switch( rb_scan_args(argc, argv, "04", &d, &p, &t, &title) ){
  case 0:
    chooser = new RBFLFile_Chooser(NULL, NULL, 0, NULL);
    break;
  case 1:
    chooser = new RBFLFile_Chooser(STR2CPTR(d), NULL, 0, NULL);
    break;
  case 2:
    chooser = new RBFLFile_Chooser(STR2CPTR(d), STR2CPTR(p), 0, NULL);
    break;
  case 3:
    chooser = new RBFLFile_Chooser(STR2CPTR(d), STR2CPTR(p), NUM2INT(t), NULL);
    break;
  case 4:
    chooser = new RBFLFile_Chooser(STR2CPTR(d), STR2CPTR(p),
				   NUM2INT(t), STR2CPTR(title));
    break;
  };
  obj = rb_FLFile_Chooser_new(klass,chooser);
  rb_obj_call_init(obj,argc,argv);
  return obj;
};

static VALUE rb_FLFile_Chooser_initialize(int argc, VALUE argv[], VALUE self)
{
  return Qnil;
};

static VALUE rb_FLFile_Chooser_destroy(VALUE self)
{
  RBFLFile_Chooser *p = (RBFLFile_Chooser*)rb_to_fl(self);

  rb_fltk_objmap_delete((Fl_Widget*)p);
  return Qnil;
};

static void rb_flfilechooser_callback_func(Fl_File_Chooser *widget, void *userdata)
{
  VALUE obj;
  VALUE callback, data;

  obj = rb_fltk_objmap_aref((Fl_Widget*)widget);
  if( obj == Qnil ) return ;

  callback = rb_ivar_get(obj, rb_id_icallback);

  if( callback != Qnil ){
    rb_funcall(callback, rb_id_call, 2, obj, CPTR2VAL(userdata));
  };
}

static VALUE rb_FLFile_Chooser_callback(int argc, VALUE argv[], VALUE self)
{
  VALUE proc, data;
  RBFLFile_Chooser *widget;
  int use_data = 0;

  if( rb_block_given_p() ){
    proc = rb_block_proc();
    switch( rb_scan_args(argc, argv, "01", &data) ){
    case 0:
      use_data = 0;
      break;
    case 1:
      use_data = 1;
      break;
    };
  }
  else{
    switch( rb_scan_args(argc, argv, "11", &proc, &data) ){
    case 1:
      use_data = 0;
      break;
    case 2:
      use_data = 1;
      break;
    };
  };

  rb_ivar_set(self, rb_id_icallback, proc);
  widget = (RBFLFile_Chooser*)rb_to_fl(self);
  if( use_data ){
    widget->callback(rb_flfilechooser_callback_func, VAL2CPTR(data));
  }
  else{
    widget->callback(rb_flfilechooser_callback_func, NULL);
  };

  return Qnil;
}

static VALUE rb_FLFile_Chooser_value(int argc, VALUE argv[], VALUE self)
{
  RBFLFile_Chooser *widget = (RBFLFile_Chooser*)rb_to_fl(self);
  VALUE arg0;

  if( rb_scan_args(argc, argv, "01", &arg0) == 0 ){
    return CPTR2STR(widget->value());
  }
  else{
    switch( TYPE(arg0) ){
    case T_FIXNUM:
    case T_BIGNUM:
      return CPTR2STR(widget->value(NUM2INT(arg0)));
    default:
      widget->value(STR2CPTR(arg0));
      return Qnil;
    }
  }
}

#ifdef HAVE_FL_FILE_CHOOSER_NEWBUTTON
static VALUE rb_FLFile_Chooser_new_button(VALUE self)
{
  RBFLFile_Chooser *widget = (RBFLFile_Chooser*)rb_to_fl(self);
  return rb_fl_new(rb_cFLButton, widget->newButton);
}
#endif

#ifdef HAVE_FL_FILE_CHOOSER_PREVIEWBUTTON
static VALUE rb_FLFile_Chooser_preview_button(VALUE self)
{
  RBFLFile_Chooser *widget = (RBFLFile_Chooser*)rb_to_fl(self);
  return rb_fl_new(rb_cFLCheck_Button, widget->previewButton);
}
#endif

extern "C" void Init_flfile(void)
{
  rb_cFLFile_Browser = rb_define_class_under(rb_mFLTK, "FileBrowser",
					     rb_cFLBrowser);
  DEF_S_METHOD(File_Browser,new,-1);
  DEF_WIDGET_METHODS(File_Browser);
  DEF_METHOD(File_Browser,load,1);
  DEF_METHOD(File_Browser,filetype,-1);
  DEF_METHOD(File_Browser,filter,-1);
  DEF_METHOD(File_Browser,iconsize,-1);

  DEF_FILE_BROWSER_CONST_NUM(FILES);
  DEF_FILE_BROWSER_CONST_NUM(DIRECTORIES);

  rb_cFLFile_Chooser = rb_define_class_under(rb_mFLTK, "FileChooser",
					     rb_cObject);
  DEF_S_METHOD(File_Chooser,new,-1);
  DEF_INIT_METHOD(File_Chooser);
  DEF_METHOD(File_Chooser,destroy,0);
  DEF_PREFIXED_METHOD_EQ(File_Chooser,file_chooser,type,-1);
  DEF_METHOD(File_Chooser,callback,-1);
  DEF_METHOD(File_Chooser,show,0);
  DEF_METHOD_Q(File_Chooser,shown,0);
  DEF_METHOD(File_Chooser,hide,0);
  DEF_METHOD_EQ(File_Chooser,filter,-1);
  DEF_METHOD_EQ(File_Chooser,textfont,-1);
  DEF_METHOD_EQ(File_Chooser,textsize,-1);
  DEF_METHOD_EQ(File_Chooser,value,-1);
#ifdef HAVE_FL_FILE_CHOOSER_FILTER_VALUE
  DEF_METHOD_EQ(File_Chooser,filter_value,-1);
#endif
  DEF_METHOD_EQ(File_Chooser,color,-1);
  DEF_METHOD(File_Chooser,count,0);
  DEF_METHOD_EQ(File_Chooser,iconsize,-1);
  DEF_METHOD_EQ(File_Chooser,label,-1);
  DEF_METHOD(File_Chooser,rescan,0);
  DEF_METHOD_Q(File_Chooser,visible,0);
#ifdef HAVE_FL_FILE_CHOOSER_NEWBUTTON
  DEF_METHOD(File_Chooser,new_button,0);
#endif
#ifdef HAVE_FL_FILE_CHOOSER_PREVIEWBUTTON
  DEF_METHOD(File_Chooser,preview_button,0);
#endif

  DEF_FILE_CHOOSER_CONST_NUM(SINGLE);
  DEF_FILE_CHOOSER_CONST_NUM(MULTI);
  DEF_FILE_CHOOSER_CONST_NUM(CREATE);
  DEF_FILE_CHOOSER_CONST_NUM(DIRECTORY);

  DEF_MODULE_CONST_NUM(PATH_MAX);
};

#endif /* HAVE_FL_FL_FILE_CHOOSER_H */
