/* -*- C++ -*-
 * $Id: flinput.cc,v 1.19 2006/12/22 22:33:27 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flinput.h"
#include "flenum.h"
#include "flcolor.h"

VALUE rb_cFLInput_;
VALUE rb_cFLInput;
VALUE rb_cFLFloat_Input;
VALUE rb_cFLInt_Input;
VALUE rb_cFLMultiline_Input;
VALUE rb_cFLSecret_Input;
VALUE rb_cFLOutput;
VALUE rb_cFLMultiline_Output;

static FN_S_NEW(Input_);
WIDGET_FNS(Input_);
static FN_GETSET_INT(Input_,mark);
#if defined(HAVE_FL_INPUT_COPY_1)
static FN_GET_INT_SET_INT(Input_,copy);
#else
static FN_GET_INT(Input_,copy);
#endif
static FN_GET_INT(Input_,undo);
static FN_GET_INT(Input_,copy_cuts);
static FN_GET_INT(Input_,size);
static FN_GET_INT_SET_INT(Input_,index);
static FN_GETSET_INT(Input_,maximum_size);
static FN_GETSET_STR(Input_,value);
static FN_GETSET_INT(Input_,textsize);
static FN_GETSET_COLOR(Input_,textcolor);
static FN_GETSET_INT(Input_,textfont);
static FN_GETSET_COLOR(Input_,cursor_color);

static VALUE rb_FLInput__position(int argc, VALUE argv[], VALUE self)
{
  RBFLInput_ *widget = (RBFLInput_*)rb_to_fl(self);
  VALUE arg1, arg2;
  int   ret;

  switch( rb_scan_args(argc, argv, "02", &arg1, &arg2) ){
  case 0:
    return INT2NUM(widget->position());
  case 1:
    ret = widget->position(NUM2INT(arg1));
    return INT2BOOL(ret);
  case 2:
    ret = widget->position(NUM2INT(arg1), NUM2INT(arg2));
    return INT2BOOL(ret);
  };
};

static VALUE rb_FLInput__cut(int argc, VALUE argv[], VALUE self)
{
  RBFLInput_ *widget = (RBFLInput_*)rb_to_fl(self);
  VALUE arg1, arg2;
  int ret;

  switch( rb_scan_args(argc, argv, "02", &arg1, &arg2) ){
  case 0:
    return INT2NUM(widget->cut());
  case 1:
    ret = widget->cut(NUM2INT(arg1));
    return INT2NUM(ret);
  case 2:
    ret = widget->cut(NUM2INT(arg1), NUM2INT(arg2));
    return INT2NUM(ret);
  };
};

static VALUE rb_FLInput__insert(VALUE self, VALUE arg1)
{
  RBFLInput_ *widget = (RBFLInput_*)rb_to_fl(self);
  int ret;

  if( arg1 == Qnil ){
    ret = widget->insert(NULL);
  }
  else{
    Check_Type(arg1, T_STRING);
    ret = widget->insert(STR2CSTR(arg1), RSTRING(arg1)->len);
  };
  return INT2NUM(ret);
};

static VALUE rb_FLInput__replace(VALUE self, VALUE arg1, VALUE arg2, VALUE arg3)
{
  RBFLInput_ *widget = (RBFLInput_*)rb_to_fl(self);
  int ret;

  if( arg3 == Qnil ){
    ret = widget->replace(NUM2INT(arg1), NUM2INT(arg2), NULL);
  }
  else{
    Check_Type(arg3, T_STRING);
    ret = widget->replace(NUM2INT(arg1), NUM2INT(arg2),
			  STR2CSTR(arg3), RSTRING(arg3)->len);
  };
  return INT2NUM(ret);
};

static FN_S_NEW(Input);
WIDGET_FNS(Input);

static FN_S_NEW(Float_Input);
WIDGET_FNS(Float_Input);

static FN_S_NEW(Int_Input);
WIDGET_FNS(Int_Input);

static FN_S_NEW(Multiline_Input);
WIDGET_FNS(Multiline_Input);

static FN_S_NEW(Secret_Input);
WIDGET_FNS(Secret_Input);

static FN_S_NEW(Output);
WIDGET_FNS(Output);

static FN_S_NEW(Multiline_Output);
WIDGET_FNS(Multiline_Output);

extern "C" void
Init_flinput(){
  rb_cFLInput_ = rb_define_class_under(rb_mFLTK, "Input_", rb_cFLWidget);
  DEF_WIDGET_METHODS(Input_);
  DEF_S_METHOD(Input_,new,-1);
  DEF_METHOD_EQ(Input_,mark,-1);
  DEF_METHOD_EQ(Input_,position,-1);
  DEF_METHOD_EQ(Input_,value,-1);
  DEF_METHOD(Input_,cut,-1);
#if defined(HAVE_FL_INPUT_COPY_1)
  DEF_METHOD(Input_,copy,1);
#else
  DEF_METHOD(Input_,copy,0);
#endif
  DEF_METHOD(Input_,undo,0);
  DEF_METHOD(Input_,copy_cuts,0);
  DEF_METHOD(Input_,insert,1);
  DEF_METHOD(Input_,replace,3);
  DEF_METHOD_EQ(Input_,maximum_size,-1);
  DEF_METHOD_EQ(Input_,textsize,-1);
  DEF_METHOD_EQ(Input_,textfont,-1);
  DEF_METHOD_EQ(Input_,textcolor,-1);
  DEF_METHOD_EQ(Input_,cursor_color,-1);
  DEF_METHOD(Input_,index,1);
  DEF_METHOD(Input_,size,0);

  rb_cFLInput = rb_define_class_under(rb_mFLTK, "Input", rb_cFLInput_);
  DEF_S_METHOD(Input,new,-1);
  DEF_WIDGET_METHODS(Input);

  rb_cFLFloat_Input = rb_define_class_under(rb_mFLTK, "FloatInput", rb_cFLInput);
  DEF_S_METHOD(Float_Input,new,-1);
  DEF_WIDGET_METHODS(Float_Input);

  rb_cFLInt_Input = rb_define_class_under(rb_mFLTK, "IntInput", rb_cFLInput);
  DEF_S_METHOD(Int_Input,new,-1);
  DEF_WIDGET_METHODS(Int_Input);

  rb_cFLMultiline_Input = rb_define_class_under(rb_mFLTK, "MultilineInput", rb_cFLInput);
  DEF_S_METHOD(Multiline_Input,new,-1);
  DEF_WIDGET_METHODS(Multiline_Input);

  rb_cFLSecret_Input = rb_define_class_under(rb_mFLTK, "SecretInput", rb_cFLInput);
  DEF_S_METHOD(Secret_Input,new,-1);
  DEF_WIDGET_METHODS(Secret_Input);

  rb_cFLOutput = rb_define_class_under(rb_mFLTK, "Output", rb_cFLInput_);
  DEF_S_METHOD(Output,new,-1);
  DEF_WIDGET_METHODS(Output);

  rb_cFLMultiline_Output = rb_define_class_under(rb_mFLTK, "MultilineOutput", rb_cFLOutput);
  DEF_S_METHOD(Multiline_Output,new,-1);
  DEF_WIDGET_METHODS(Multiline_Output);

  DEF_MODULE_CONST_NUM(FLOAT_INPUT);
  DEF_MODULE_CONST_NUM(HIDDEN_INPUT);
  DEF_MODULE_CONST_NUM(INPUT_READONLY);
  DEF_MODULE_CONST_NUM(INPUT_TYPE);
  DEF_MODULE_CONST_NUM(INPUT_WRAP);
  DEF_MODULE_CONST_NUM(INT_INPUT);
  DEF_MODULE_CONST_NUM(MULTILINE_INPUT);
  DEF_MODULE_CONST_NUM(MULTILINE_INPUT_WRAP);
  DEF_MODULE_CONST_NUM(MULTILINE_OUTPUT);
  DEF_MODULE_CONST_NUM(MULTILINE_OUTPUT_WRAP);
  DEF_MODULE_CONST_NUM(NORMAL_INPUT);
  DEF_MODULE_CONST_NUM(NORMAL_OUTPUT);
  DEF_MODULE_CONST_NUM(SECRET_INPUT);
};
