/* -*- C++ -*-
 * $Id: flmenu.cc,v 1.46 2006/12/31 00:36:33 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flmenu.h"
#include "flenum.h"
#include "flcolor.h"

#define DEF_MENU_BUTTON_CONST_NUM(c) \
  rb_define_const(rb_cFLMenu_Button,#c,INT2NUM(Fl_Menu_Button::c))

VALUE rb_cFLMenu_;
VALUE rb_cFLChoice;
VALUE rb_cFLMenu_Bar;
VALUE rb_cFLMenu_Button;

VALUE rb_fltk_menumap;

static void rb_flmenu_mark(VALUE obj)
{
  RBFLMenu_ *w = (RBFLMenu_*)rb_to_fl(obj);
  const Fl_Menu_Item *item = w->menu();
  int sub;
  if( item ){
    sub = 0;
    for( int i=0; (sub > 0) || item[i].text; i++ ){
      rb_gc_mark(CPTR2VAL(item[i].user_data_));
      if( item[i].flags & FL_SUBMENU ){
	sub ++;
      }
      if( !item[i].text ){
	sub --;
      }
    }
  }
}

static VALUE rb_flmenu_item2ary(const Fl_Menu_Item *item)
{
  VALUE ary;
  int sub;

  if( !item ){
    return Qnil;
  }

  ary = rb_ary_new();
  sub = 0;
  for( int i=0; (sub > 0) || item[i].text; i++ ){
    VALUE callback, user_data;
    callback = rb_ary_entry(CPTR2VAL(item[i].user_data_), 0);
    user_data = rb_ary_entry(CPTR2VAL(item[i].user_data_), 1);
    rb_ary_push(ary,
		rb_ary_new3(9,
			    CPTR2STR(item[i].text),
			    INT2NUM(item[i].shortcut_),
			    callback,
			    user_data,
			    INT2NUM(item[i].flags),
			    INT2NUM(item[i].labeltype_),
			    INT2NUM(item[i].labelfont_),
			    INT2NUM(item[i].labelsize_),
			    COLOR2VALUE(item[i].labelcolor_)));
    if( item[i].flags & FL_SUBMENU ){
      sub ++;
    }
    if( !item[i].text ){
      sub --;
    }
  }

  return ary;
}

static void rb_flmenu_internal_func(Fl_Widget *widget, void *user_data)
{
  if( user_data ){
    VALUE obj = fl_to_rb(widget);
    VALUE proc = rb_ary_entry(CPTR2VAL(user_data), 0);
    VALUE data = rb_ary_entry(CPTR2VAL(user_data), 1);
    if( proc != Qnil ){
      rb_funcall(proc, rb_id_call, 2, obj, data);
    }
  }
}

static Fl_Menu_Item* rb_flmenu_ary2item(VALUE ary)
{
  Fl_Menu_Item *item;

  if( ary == Qnil ){
    return NULL;
  }

  VALUE val = rb_hash_aref(rb_fltk_menumap, ary);
  if( val != Qnil ){
    return (Fl_Menu_Item*)NUM2INT(val);
  }

  item = new Fl_Menu_Item[RARRAY(ary)->len + 1];
  rb_hash_aset(rb_fltk_menumap, ary, INT2NUM((long)item));

  for( int i = 0; i < RARRAY(ary)->len; i++ ){
    VALUE entry = rb_ary_entry(ary, i);
    item[i].text = STR2CPTR(rb_ary_entry(entry, 0));
    item[i].shortcut_ = NUM2INT(rb_Integer(rb_ary_entry(entry, 1)));
    item[i].callback_ = rb_flmenu_internal_func;
    item[i].user_data_ = (void*)rb_ary_new3(2,
					    rb_ary_entry(entry, 2),
					    rb_ary_entry(entry, 3));
    item[i].flags = NUM2INT(rb_Integer(rb_ary_entry(entry, 4)));
    item[i].labeltype_ = NUM2INT(rb_Integer(rb_ary_entry(entry, 5)));
    item[i].labelfont_ = NUM2INT(rb_Integer(rb_ary_entry(entry, 6)));
    item[i].labelsize_ = NUM2INT(rb_Integer(rb_ary_entry(entry, 7)));
#ifdef USE_COLOR_CLASSES
    item[i].labelcolor_ = VALUE2COLOR(rb_ary_entry(entry, 8));
#else // #ifdef USE_COLOR_CLASSES 
    item[i].labelcolor_ = NUM2INT(rb_Integer(rb_ary_entry(entry, 8)));
#endif // #ifdef USE_COLOR_CLASSES 
  }
  item[RARRAY(ary)->len].text = NULL;

  return item;
}

static FN_S_NEW(Menu_);
WIDGET_FNS(Menu_);
static FN_INIT(Menu_);
static FN_GETSET_INT(Menu_,value);
static FN_SET_INT2(Menu_,shortcut);
static FN_SET_INT_OPTINT(Menu_,mode);
static FN_GET_STR_SET_OPTINT(Menu_,text);
static FN_SET_INT(Menu_, remove);
static FN_CALL(Menu_, clear);
static FN_GETSET_ENUM(Menu_, Fl_Font, textfont);
static FN_GETSET_COLOR(Menu_, textcolor);
static FN_GETSET_INT(Menu_, textsize);


static VALUE rb_FLMenu__add(int argc, VALUE argv[], VALUE self)
{
  RBFLMenu_ *widget = (RBFLMenu_*)rb_to_fl(self);
  VALUE label, shortcut, proc, data, flags;
  VALUE pass_data;
  int ret = 0;

  if( rb_block_given_p() ){
    proc = rb_block_proc();
    switch( rb_scan_args(argc, argv, "13", &label, &shortcut, &data, &flags) ){
    case 1:
      shortcut = Qnil;
      pass_data = rb_assoc_new(proc, Qnil);
      flags = INT2NUM(0);
    case 2:
      pass_data = rb_assoc_new(proc, Qnil);
      flags = INT2NUM(0);
      break;
    case 3:
      pass_data = rb_assoc_new(proc, data);
      flags = INT2NUM(0);
      break;
    case 4:
      pass_data = rb_assoc_new(proc, data);
      break;
    };
  }
  else{
    if( argc == 1 ){
      char *text = STR2CSTR(argv[0]);
      return INT2NUM(widget->add(text));
    };

    switch( rb_scan_args(argc,argv,"32",&label,&shortcut,&proc,&data,&flags) ){
    case 3:
      pass_data = rb_assoc_new(proc, Qnil);
      flags = INT2NUM(0);
      break;
    case 4:
      pass_data = rb_assoc_new(proc, data);
      flags = INT2NUM(0);
      break;
    case 5:
      pass_data = rb_assoc_new(proc, data);
      break;
    };
  };

  if( TYPE(shortcut) == T_STRING ){
    ret = widget->add(STR2CSTR(label), STR2CSTR(shortcut),
		      rb_flmenu_internal_func, VAL2CPTR(pass_data),
		      NUM2INT(flags));
  }
  else if( shortcut == Qnil ){
    ret = widget->add(STR2CSTR(label), 0,
		      rb_flmenu_internal_func, VAL2CPTR(pass_data),
		      NUM2INT(flags));
  }
  else{
    ret = widget->add(STR2CSTR(label), NUM2INT(shortcut),
		      rb_flmenu_internal_func, VAL2CPTR(pass_data),
		      NUM2INT(flags));
  };
  return INT2NUM(ret);
}

static VALUE rb_FLMenu__menu(int argc, VALUE argv[], VALUE self)
{
  RBFLMenu_ *widget = (RBFLMenu_*)rb_to_fl(self);
  VALUE ary;
  VALUE ret = Qnil;

  switch( rb_scan_args(argc, argv, "01", &ary) ){
  case 0:
    ret = rb_flmenu_item2ary(widget->menu());
    break;
  case 1:
    widget->menu(rb_flmenu_ary2item(ary));
    ret = Qnil;
    break;
  }

  return ret;
}

static FN_S_NEW(Choice);
WIDGET_FNS(Choice);

static FN_S_NEW(Menu_Bar);
WIDGET_FNS(Menu_Bar);

static FN_S_NEW(Menu_Button);
WIDGET_FNS(Menu_Button);
static FN_GET_WIDGET(Menu_Button,popup);
static FN_GETSET_INT(Menu_Button,type);

extern "C" void
Init_flmenu(){
  rb_cFLMenu_ = rb_define_class_under(rb_mFLTK, "Menu_", rb_cFLWidget);
  rb_fltk_append_mark(rb_cFLMenu_, rb_flmenu_mark);
  rb_define_const(rb_mFLTK, "Menu", rb_cFLMenu_);
  DEF_S_METHOD(Menu_,new,-1);
  DEF_WIDGET_METHODS(Menu_);
  DEF_INIT_METHOD(Menu_);
  DEF_METHOD(Menu_,add,-1);
  DEF_METHOD(Menu_,remove,1);
  DEF_METHOD(Menu_,clear,0);
  DEF_METHOD_EQ(Menu_,value,-1);
  DEF_METHOD(Menu_,shortcut,2);
  DEF_METHOD_EQ(Menu_,mode,-1);
  DEF_METHOD_EQ(Menu_,text,-1);
  DEF_METHOD_EQ(Menu_,menu,-1);
  DEF_METHOD_EQ(Menu_,textfont,-1);
  DEF_METHOD_EQ(Menu_,textcolor,-1);
  DEF_METHOD_EQ(Menu_,textsize,-1);

  DEF_MODULE_CONST_NUM(MENU_INACTIVE);
  DEF_MODULE_CONST_NUM(MENU_TOGGLE);
  DEF_MODULE_CONST_NUM(MENU_VALUE);
  DEF_MODULE_CONST_NUM(MENU_RADIO);
  DEF_MODULE_CONST_NUM(MENU_INVISIBLE);
  DEF_MODULE_CONST_NUM(SUBMENU_POINTER);
  DEF_MODULE_CONST_NUM(SUBMENU);
  DEF_MODULE_CONST_NUM(MENU_DIVIDER);
  DEF_MODULE_CONST_NUM(MENU_HORIZONTAL);

  rb_fltk_menumap = rb_hash_new();
  rb_define_const(rb_cFLMenu_, "MENUMAP", rb_fltk_menumap);

  rb_cFLChoice = rb_define_class_under(rb_mFLTK, "Choice", rb_cFLMenu_);
  DEF_S_METHOD(Choice,new,-1);
  DEF_WIDGET_METHODS(Choice);

  rb_cFLMenu_Bar = rb_define_class_under(rb_mFLTK, "MenuBar", rb_cFLMenu_);
  DEF_S_METHOD(Menu_Bar,new,-1);
  DEF_WIDGET_METHODS(Menu_Bar);

  rb_cFLMenu_Button = rb_define_class_under(rb_mFLTK, "MenuButton", rb_cFLMenu_);
  DEF_S_METHOD(Menu_Button,new,-1);
  DEF_WIDGET_METHODS(Menu_Button);
  DEF_METHOD(Menu_Button,popup,0);
  DEF_PREFIXED_METHOD_EQ(Menu_Button,menu_button,type,-1);

  DEF_MENU_BUTTON_CONST_NUM(POPUP1);
  DEF_MENU_BUTTON_CONST_NUM(POPUP2);
  DEF_MENU_BUTTON_CONST_NUM(POPUP3);
  DEF_MENU_BUTTON_CONST_NUM(POPUP12);
  DEF_MENU_BUTTON_CONST_NUM(POPUP13);
  DEF_MENU_BUTTON_CONST_NUM(POPUP23);
  DEF_MENU_BUTTON_CONST_NUM(POPUP123);

}
