/* -*- C++ -*-
 * $Id: fltooltip.cc,v 1.7 2006/12/29 05:26:14 jeremy_henty Exp $
 */

#ifdef HAVE_FL_FL_TOOLTIP_H

#include "fltk.h"
#include "fltooltip.h"
#include "flcolor.h"
#include "flwidget.h"

VALUE rb_mFLTooltip;

static VALUE rb_fltk_tooltip_delay(int argc, VALUE argv[], VALUE module)
{
  VALUE ret, arg0;

  switch( rb_scan_args(argc, argv, "01", &arg0) ){
  case 0:
    ret = rb_float_new(Fl_Tooltip::delay());
    break;
  case 1:
    Fl_Tooltip::delay(RFLOAT(rb_Float(arg0))->value);
    ret = Qnil;
  }
  return ret;
}

static VALUE rb_fltk_tooltip_enable(VALUE module)
{
  Fl_Tooltip::enable();
  return Qnil;
}

static VALUE rb_fltk_tooltip_disable(VALUE module)
{
  Fl_Tooltip::disable();
  return Qnil;
}

static VALUE rb_fltk_tooltip_enter_value;
static VALUE rb_fltk_tooltip_exit_value;

static void rb_fltk_tooltip_enter_func(Fl_Widget *w)
{
  if( rb_fltk_tooltip_enter_value != Qnil ){
    rb_funcall(rb_fltk_tooltip_enter_value, rb_intern("call"), 1, fl_to_rb(w));
  }
}

static void rb_fltk_tooltip_exit_func(Fl_Widget *w)
{
  if( rb_fltk_tooltip_exit_value != Qnil ){
    rb_funcall(rb_fltk_tooltip_exit_value, rb_intern("call"), 1, fl_to_rb(w));
  }
}

static VALUE rb_fltk_tooltip_enter(int argc, VALUE argv[], VALUE module)
{
  VALUE proc;

  if( rb_block_given_p() ){
    rb_scan_args(argc, argv, "0");
    rb_fltk_tooltip_enter_value = rb_block_proc();
    return Qnil;
  }
  else{
    switch( rb_scan_args(argc, argv, "01", &proc) ){
    case 0:
      return rb_fltk_tooltip_enter_value;
    case 1:
      rb_fltk_tooltip_enter_value = proc;
      return Qnil;
    }
  }
}

static VALUE rb_fltk_tooltip_exit(int argc, VALUE argv[], VALUE module)
{
  VALUE proc;

  if( rb_block_given_p() ){
    rb_scan_args(argc, argv, "0");
    rb_fltk_tooltip_exit_value = rb_block_proc();
    return Qnil;
  }
  else{
    switch( rb_scan_args(argc, argv, "01", &proc) ){
    case 0:
      return rb_fltk_tooltip_exit_value;
    case 1:
      rb_fltk_tooltip_exit_value = proc;
      return Qnil;
    }
  }
}

static VALUE rb_fltk_tooltip_color(int argc, VALUE argv[], VALUE module)
{
  VALUE ret, arg0;

  switch( rb_scan_args(argc, argv, "01", &arg0) ){
  case 0:
    ret = COLOR2VALUE(Fl_Tooltip::color());
    break;
  case 1:
    Fl_Tooltip::color(VALUE2COLOR(arg0));
    ret = Qnil;
    break;
  }
  return ret;
}

#ifdef HAVE_FL_TOOLTIP_TEXTCOLOR
static VALUE rb_fltk_tooltip_textcolor(int argc, VALUE argv[], VALUE module)
{
  VALUE ret, arg0;

  switch( rb_scan_args(argc, argv, "01", &arg0) ){
  case 0:
    ret = COLOR2VALUE(Fl_Tooltip::textcolor());
    break;
  case 1:
    Fl_Tooltip::textcolor(VALUE2COLOR(arg0));
    ret = Qnil;
    break;
  }
  return ret;
}
#endif

static VALUE rb_fltk_tooltip_font(int argc, VALUE argv[], VALUE module)
{
  VALUE ret, arg0;

  switch( rb_scan_args(argc, argv, "01", &arg0) ){
  case 0:
    ret = INT2NUM(Fl_Tooltip::font());
    break;
  case 1:
    Fl_Tooltip::font(NUM2INT(arg0));
    ret = Qnil;
    break;
  }
  return ret;
}

static VALUE rb_fltk_tooltip_size(int argc, VALUE argv[], VALUE module)
{
  VALUE ret, arg0;

  switch( rb_scan_args(argc, argv, "01", &arg0) ){
  case 0:
    ret = INT2NUM(Fl_Tooltip::size());
    break;
  case 1:
    Fl_Tooltip::size(NUM2INT(arg0));
    ret = Qnil;
    break;
  }
  return ret;
}

extern "C" void
Init_fltooltip(){
  rb_fltk_tooltip_enter_value = Qnil;
  rb_fltk_tooltip_exit_value = Qnil;

  rb_mFLTooltip = rb_define_module_under(rb_mFLTK, "Tooltip");
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "delay", rb_fltk_tooltip_delay, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "delay=", rb_fltk_tooltip_delay, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "enable", rb_fltk_tooltip_enable, 0);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "disable", rb_fltk_tooltip_disable, 0);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "enter", rb_fltk_tooltip_enter, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "enter=", rb_fltk_tooltip_enter, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "exit", rb_fltk_tooltip_exit, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "exit=", rb_fltk_tooltip_exit, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "color", rb_fltk_tooltip_color, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "color=", rb_fltk_tooltip_color, -1);
#ifdef HAVE_FL_TOOLTIP_TEXTCOLOR
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "textcolor", rb_fltk_tooltip_textcolor, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "textcolor=", rb_fltk_tooltip_textcolor, -1);
#endif
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "font", rb_fltk_tooltip_font, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "font=", rb_fltk_tooltip_font, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "size", rb_fltk_tooltip_size, -1);
  RB_DEF_MODULE_FUNC(rb_mFLTooltip, "size=", rb_fltk_tooltip_size, -1);
};
#endif
