/* -*- C++ -*-
 * $Id: flvaluator.cc,v 1.19 2006/12/21 16:47:34 jeremy_henty Exp $
 */

#include "fltk.h"
#include "flvaluator.h"
#include "flenum.h"
#include "flcolor.h"

VALUE rb_cFLValuator;
VALUE rb_cFLAdjuster;
VALUE rb_cFLCounter;
VALUE rb_cFLDial;
VALUE rb_cFLRoller;
VALUE rb_cFLSlider;
VALUE rb_cFLFill_Slider;
VALUE rb_cFLNice_Slider;
VALUE rb_cFLHor_Slider;
VALUE rb_cFLHor_Fill_Slider;
VALUE rb_cFLHor_Nice_Slider;
VALUE rb_cFLScrollbar;
VALUE rb_cFLValue_Slider;
VALUE rb_cFLValue_Input;
VALUE rb_cFLValue_Output;

static FN_S_NEW(Valuator);
WIDGET_FNS(Valuator);
static FN_GETSET_DBL(Valuator,value);
static FN_GETSET_DBL(Valuator,minimum);
static FN_GETSET_DBL(Valuator,maximum);
static FN_SET_DBL2(Valuator,range);
static FN_GET_DBL_SET_DBL(Valuator,round);
static FN_GET_DBL_SET_DBL(Valuator,clamp);
static FN_GET_DBL_SET_DBL_INT(Valuator,increment);
static FN_SET_DBL2(Valuator,bounds);
static FN_GET_INT_SET_STR(Valuator,format);
static FN_GET_BOOL(Valuator,changed);
static FN_CALL(Valuator,set_changed);
static FN_CALL(Valuator,clear_changed);

static VALUE rb_FLValuator_step(int argc, VALUE argv[], VALUE self)
{
  RBFLValuator *widget = (RBFLValuator*)(rb_to_fl(self));
  VALUE arg1, arg2;

  switch( rb_scan_args(argc, argv, "02", &arg1, &arg2) ){
  case 0:
    return rb_float_new(widget->step());
  case 1:
    switch( TYPE(arg1) ){
    case T_FIXNUM:
    case T_BIGNUM:
      widget->step((int)NUM2INT(arg1));
      return Qnil;
    default:
      arg1 = rb_Float(arg1);
      widget->step(RFLOAT(arg1)->value);
      return Qnil;
    };
  case 2:
    arg1 = rb_Float(arg1);
    widget->step(RFLOAT(arg1)->value, FIX2INT(arg2));
    return Qnil;
  };
};

static FN_S_NEW(Adjuster);
WIDGET_FNS(Adjuster);
static FN_GETSET_BOOL(Adjuster,soft);

static FN_S_NEW(Counter);
WIDGET_FNS(Counter);
static FN_GETSET_ENUM(Counter, Fl_Font, textfont);
static FN_GETSET_COLOR(Counter, textcolor);
static FN_GETSET_INT(Counter, textsize);
static FN_SET_DBL(Counter, lstep);

static FN_S_NEW(Dial);
WIDGET_FNS(Dial);
static FN_GETSET_INT(Dial, angle1);
static FN_GETSET_INT(Dial, angle2);
static FN_SET_INT2(Dial, angles);

static FN_S_NEW(Roller);
WIDGET_FNS(Roller);

static FN_S_NEW(Slider);
WIDGET_FNS(Slider);
static FN_GET_INT_SET_INT4(Slider,scrollvalue);
static FN_GETSET_ENUM(Slider, Fl_Boxtype, slider);
static FN_GETSET_DBL(Slider, slider_size);

static FN_S_NEW(Nice_Slider);
WIDGET_FNS(Nice_Slider);

static FN_S_NEW(Fill_Slider);
WIDGET_FNS(Fill_Slider);

static FN_S_NEW(Hor_Slider);
WIDGET_FNS(Hor_Slider);

static FN_S_NEW(Hor_Fill_Slider);
WIDGET_FNS(Hor_Fill_Slider);

static FN_S_NEW(Hor_Nice_Slider);
WIDGET_FNS(Hor_Nice_Slider);

static FN_S_NEW(Scrollbar);
WIDGET_FNS(Scrollbar);
static FN_GETSET_INT(Scrollbar, linesize);

static FN_S_NEW(Value_Slider);
WIDGET_FNS(Value_Slider);
static FN_GETSET_ENUM(Value_Slider, Fl_Font, textfont);
static FN_GETSET_COLOR(Value_Slider, textcolor);
static FN_GETSET_INT(Value_Slider, textsize);

static FN_S_NEW(Value_Input);
WIDGET_FNS(Value_Input);
static FN_GETSET_ENUM(Value_Input, Fl_Font, textfont);
static FN_GETSET_COLOR(Value_Input, textcolor);
static FN_GETSET_INT(Value_Input, textsize);
static FN_GETSET_BOOL(Value_Input, soft);

static FN_S_NEW(Value_Output);
WIDGET_FNS(Value_Output);
static FN_GETSET_ENUM(Value_Output, Fl_Font, textfont);
static FN_GETSET_COLOR(Value_Output, textcolor);
static FN_GETSET_INT(Value_Output, textsize);
static FN_GETSET_BOOL(Value_Output, soft);

extern "C" void
Init_flvaluator(){
  rb_cFLValuator = rb_define_class_under(rb_mFLTK, "Valuator", rb_cFLWidget);
  DEF_S_METHOD(Valuator,new,-1);
  DEF_WIDGET_METHODS(Valuator);
  DEF_METHOD_EQ(Valuator,value,-1);
  DEF_METHOD_EQ(Valuator,minimum,-1);
  DEF_METHOD_EQ(Valuator,maximum,-1);
  DEF_METHOD(Valuator,range,2);
  DEF_METHOD_EQ(Valuator,step,-1);
  DEF_METHOD(Valuator,round,-1);
  DEF_METHOD(Valuator,clamp,-1);
  DEF_METHOD(Valuator,increment,-1);
  DEF_METHOD(Valuator,bounds,2);
  DEF_METHOD(Valuator,format,1);
  DEF_METHOD_Q(Valuator,changed,0);
  DEF_METHOD(Valuator,set_changed,0);
  DEF_METHOD(Valuator,clear_changed,0);

  DEF_MODULE_CONST_NUM(HORIZONTAL);
  DEF_MODULE_CONST_NUM(VERTICAL);

  rb_cFLAdjuster = rb_define_class_under(rb_mFLTK, "Adjuster", rb_cFLValuator);
  DEF_S_METHOD(Adjuster,new,-1);
  DEF_WIDGET_METHODS(Adjuster);
  DEF_METHOD_QEQ(Adjuster,soft,-1);

  rb_cFLCounter = rb_define_class_under(rb_mFLTK, "Counter", rb_cFLValuator);
  DEF_S_METHOD(Counter,new,-1);
  DEF_WIDGET_METHODS(Counter);
  DEF_METHOD_EQ(Counter, textfont, -1);
  DEF_METHOD_EQ(Counter, textsize, -1);
  DEF_METHOD_EQ(Counter, textcolor, -1);
  DEF_METHOD(Counter, lstep, 1);

  DEF_MODULE_CONST_NUM(NORMAL_COUNTER);
  DEF_MODULE_CONST_NUM(SIMPLE_COUNTER);

  rb_cFLDial = rb_define_class_under(rb_mFLTK, "Dial", rb_cFLValuator);
  DEF_S_METHOD(Dial,new,-1);
  DEF_WIDGET_METHODS(Dial);
  DEF_METHOD_EQ(Dial, angle1, -1);
  DEF_METHOD_EQ(Dial, angle2, -1);
  DEF_METHOD(Dial, angles, 2);

  DEF_MODULE_CONST_NUM(NORMAL_DIAL);
  DEF_MODULE_CONST_NUM(LINE_DIAL);
  DEF_MODULE_CONST_NUM(FILL_DIAL);

  rb_cFLRoller = rb_define_class_under(rb_mFLTK, "Roller", rb_cFLValuator);
  DEF_S_METHOD(Roller,new,-1);
  DEF_WIDGET_METHODS(Roller);

  rb_cFLSlider = rb_define_class_under(rb_mFLTK, "Slider", rb_cFLValuator);
  DEF_S_METHOD(Slider,new,-1);
  DEF_WIDGET_METHODS(Slider);
  DEF_METHOD(Slider,scrollvalue,4);
  DEF_METHOD_EQ(Slider, slider, -1);  
  DEF_METHOD_EQ(Slider, slider_size, -1);

  DEF_MODULE_CONST_NUM(VERT_SLIDER);
  DEF_MODULE_CONST_NUM(HOR_SLIDER);
  DEF_MODULE_CONST_NUM(VERT_FILL_SLIDER);
  DEF_MODULE_CONST_NUM(HOR_FILL_SLIDER);
  DEF_MODULE_CONST_NUM(VERT_NICE_SLIDER);
  DEF_MODULE_CONST_NUM(HOR_NICE_SLIDER);

  rb_cFLFill_Slider = rb_define_class_under(rb_mFLTK, "FillSlider", rb_cFLSlider);
  DEF_S_METHOD(Fill_Slider,new,-1);
  DEF_WIDGET_METHODS(Fill_Slider);

  rb_cFLNice_Slider = rb_define_class_under(rb_mFLTK, "NiceSlider", rb_cFLSlider);
  DEF_S_METHOD(Nice_Slider,new,-1);
  DEF_WIDGET_METHODS(Nice_Slider);

  rb_cFLHor_Slider = rb_define_class_under(rb_mFLTK, "HorSlider", rb_cFLSlider);
  DEF_S_METHOD(Hor_Slider,new,-1);
  DEF_WIDGET_METHODS(Hor_Slider);

  rb_cFLHor_Fill_Slider = rb_define_class_under(rb_mFLTK, "HorFillSlider", rb_cFLSlider);
  DEF_S_METHOD(Hor_Fill_Slider,new,-1);
  DEF_WIDGET_METHODS(Hor_Fill_Slider);

  rb_cFLHor_Nice_Slider = rb_define_class_under(rb_mFLTK, "HorNiceSlider", rb_cFLSlider);
  DEF_S_METHOD(Hor_Nice_Slider,new,-1);
  DEF_WIDGET_METHODS(Hor_Nice_Slider);


  rb_cFLValue_Slider = rb_define_class_under(rb_mFLTK, "ValueSlider", rb_cFLSlider);
  DEF_S_METHOD(Value_Slider,new,-1);
  DEF_WIDGET_METHODS(Value_Slider);
  DEF_METHOD_EQ(Value_Slider, textfont, -1);
  DEF_METHOD_EQ(Value_Slider, textsize, -1);
  DEF_METHOD_EQ(Value_Slider, textcolor, -1);

  rb_cFLScrollbar = rb_define_class_under(rb_mFLTK, "Scrollbar", rb_cFLSlider);
  DEF_S_METHOD(Scrollbar,new,-1);
  DEF_WIDGET_METHODS(Scrollbar);
  DEF_METHOD_EQ(Scrollbar, linesize, -1);

  rb_cFLValue_Input = rb_define_class_under(rb_mFLTK, "ValueInput", rb_cFLValuator);
  DEF_S_METHOD(Value_Input,new,-1);
  DEF_WIDGET_METHODS(Value_Input);
  DEF_METHOD_EQ(Value_Input, textfont, -1);
  DEF_METHOD_EQ(Value_Input, textsize, -1);
  DEF_METHOD_EQ(Value_Input, textcolor, -1);
  DEF_METHOD_QEQ(Value_Input, soft, -1);

  rb_cFLValue_Output = rb_define_class_under(rb_mFLTK,"ValueOutput",rb_cFLValuator);
  DEF_S_METHOD(Value_Output,new,-1);
  DEF_WIDGET_METHODS(Value_Output);
  DEF_METHOD_EQ(Value_Output, textfont, -1);
  DEF_METHOD_EQ(Value_Output, textsize, -1);
  DEF_METHOD_EQ(Value_Output, textcolor, -1);
  DEF_METHOD_QEQ(Value_Output, soft, -1);
};
