/* -*- C++ -*-
 * $Id: flvlist.cc,v 1.7 2006/12/29 05:37:32 jeremy_henty Exp $
 */

#include "../fltk.h"
#include "flvlist.h"
#include "../flvaluator.h"
#include "flvstyle.h"
#include "../flenum.h"
#include "../flwidget.h"

#define DEF_FLVE_CONST_NUM(c) \
  DEF_MODULE_CONST("VE_"#c,INT2NUM(FLVE_##c))
#define DEF_FLVECB_CONST_NUM(c) \
  DEF_MODULE_CONST("VEcb_"#c,INT2NUM(FLVEcb_##c))
#define DEF_FLVF_CONST_NUM(c) \
  DEF_MODULE_CONST("VF_"#c,INT2NUM(FLVF_##c))
#define DEF_FLVS_CONST_NUM(c) \
  DEF_MODULE_CONST("VS_"#c,INT2NUM(FLVF_##c))
#define DEF_FLV_CONST_NUM(c) \
  DEF_MODULE_CONST("VS_"#c,INT2NUM(FLV_##c))

void RBFLVListInternalWidgetsMask::init_RBFLVListInternalWidgetsMask(RBFLVList *p){
	vl = p;
	rb_scrollbar = Data_Wrap_Struct(rb_cFLScrollbar,0,0,&vl->scrollbar);
	rb_fltk_objmap_aset(&vl->scrollbar, rb_scrollbar);
	rb_hscrollbar = Data_Wrap_Struct(rb_cFLScrollbar,0,0,&vl->hscrollbar);
	rb_fltk_objmap_aset(&vl->hscrollbar, rb_hscrollbar);
	rb_global_style = Data_Wrap_Struct(rb_cFLVStyle,0,0,&vl->global_style);
	rb_gc_register_address(&rb_global_style);
	rb_row_style = Data_Wrap_Struct(rb_cFLVStyleList,0,0,&vl->row_style);
	rb_gc_register_address(&rb_row_style);
};

RBFLVListInternalWidgetsMask::~RBFLVListInternalWidgetsMask(){
	if(vl){
	    rb_fltk_objmap_delete((Fl_Widget*)&vl->scrollbar);
	    rb_fltk_objmap_delete((Fl_Widget*)&vl->hscrollbar);
	    rb_gc_unregister_address(&rb_global_style);
	    rb_gc_unregister_address(&rb_row_style);
	}
};

VALUE rb_cFLVList;

static FN_S_NEW(VList);
static FN_INIT(VList);
WIDGET_FNS(VList);
static FN_GETSET_INT(VList, callback_when);
static FN_GET_INT_SET_INT(VList, add_callback_when);
static FN_GET_INT_SET_INT(VList, clear_callback_when);
static FN_GET_BOOL_SET_INT(VList,callback_on );
static FN_GETSET_INT(VList, edit_when);
static FN_CALL(VList, start_edit);
static FN_CALL(VList, end_edit);
static FN_CALL(VList, cancel_edit);
static FN_GET_WIDGET(VList, editor);
static FN_GET_INT(VList, bottom_row);
static FN_GET_INT(VList, clicks);
static FN_CALL(VList, clear_clicks);
static FN_GETSET_INT(VList, max_clicks);
//static FN_GETSET_COLOR(VList, dead_space_color);
static FN_GETSET_ENUM(VList,Flv_Feature, feature);
static FN_GET_INT_SET_ENUM(VList, Flv_Feature, feature_add);
static FN_GET_INT_SET_ENUM(VList, Flv_Feature, feature_remove);
static FN_GET_BOOL_SET_INT(VList, feature_test);
static FN_GET_BOOL(VList, row_footer);
static FN_GET_BOOL(VList, row_header);
static FN_GET_BOOL(VList, row_divider);
static FN_GET_BOOL(VList, multi_select);
static FN_GET_BOOL(VList, persist_select);
static FN_GET_BOOL(VList, full_resize);
static FN_GETSET_BOOL(VList, select_locked);
static FN_GETSET_INT(VList, row);
static FN_GETSET_INT(VList, rows);
static FN_GETSET_INT(VList, row_width);
static FN_GETSET_ENUM(VList,Flv_ShowScrollbar, has_scrollbar);

/*
static VALUE rb_FLVList_rows(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  VALUE arg1;

  fprintf(stderr,"%d ++++++++++++\n",__LINE__);

  if( rb_scan_args(argc, argv, "01", &arg1) == 1){
    return INT2FIX(widget->rows(FIX2INT(arg1)));
  }
  return INT2FIX(widget->rows());
}
*/

static VALUE rb_FLVList_global_style(VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  return widget->rb_global_style;
};

static VALUE rb_FLVList_row_style(VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  return widget->rb_row_style;
};

static VALUE rb_FLVList_scrollbar(VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  return widget->rb_scrollbar;
};

static VALUE rb_FLVList_hscrollbar(VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  return widget->rb_hscrollbar;
};

static VALUE rb_FLVList_get_style(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  VALUE arg1,arg2,arg3;

  Flv_Style *s; 
  Data_Get_Struct(argv[0],Flv_Style,s);

  switch( rb_scan_args(argc, argv, "12", &arg1, &arg2, &arg3) ){
  case 1:
    widget->call_parent_get_style(*s,0,0);
    break;
  case 2:
    widget->call_parent_get_style(*s,FIX2INT(arg2),0);
    break;
  case 3:
    widget->call_parent_get_style(*s,FIX2INT(arg2),FIX2INT(arg3));
    break;
  };
  return Qnil;
};

static VALUE rb_FLVList_get_default_style(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  VALUE arg1,arg2,arg3;

  Flv_Style *s; 
  Data_Get_Struct(argv[0],Flv_Style,s);

  widget->get_default_style(*s);

  return argv[0];
};

static VALUE rb_FLVList_save_editor(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  Fl_Widget *editor = (Fl_Widget*)rb_to_fl(argv[0]);
  VALUE arg1,arg2,arg3;

  switch( rb_scan_args(argc, argv, "21", &arg1, &arg2, &arg3) ){
  case 2:
    widget->call_parent_save_editor(editor,FIX2INT(arg2),0);
    break;
  case 3:
    widget->call_parent_load_editor(editor,FIX2INT(arg2),FIX2INT(arg3));
    break;
  };
  return Qnil;
};

static VALUE rb_FLVList_load_editor(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  Fl_Widget *editor = (Fl_Widget*)rb_to_fl(argv[0]);
  VALUE arg1,arg2,arg3;

  switch( rb_scan_args(argc, argv, "21", &arg1, &arg2, &arg3) ){
  case 2:
    widget->call_parent_load_editor(editor,FIX2INT(arg2),0);
    break;
  case 3:
    widget->call_parent_load_editor(editor,FIX2INT(arg2),FIX2INT(arg3));
    break;
  };
  return Qnil;
};

static VALUE rb_FLVList_position_editor(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  Fl_Widget *editor = (Fl_Widget*)rb_to_fl(argv[0]);    
  Flv_Style *s; 
  Data_Get_Struct(argv[5],Flv_Style,s);
  widget->call_parent_position_editor(editor,FIX2INT(argv[1]),FIX2INT(argv[2]),
    FIX2INT(argv[3]),FIX2INT(argv[4]),*s);
  return Qnil;
};

static VALUE rb_FLVList_row_height(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  VALUE arg1,arg2;
  int rc = 0;

  switch( rb_scan_args(argc, argv, "11", &arg1, &arg2) ){
  case 1:
    rc = widget->call_parent_row_height(FIX2INT(arg1));
    break;
  case 2:
    rc = widget->call_parent_row_height(FIX2INT(arg1),FIX2INT(arg2));
    break;
  };
  return INT2FIX(rc);
};

static VALUE rb_FLVList_draw_row(int argc, VALUE argv[], VALUE self)
{
  RBFLVList *widget = (RBFLVList*)rb_to_fl(self);
  int x = FIX2INT(argv[1]),
      y = FIX2INT(argv[2]),
      w = FIX2INT(argv[3]),
      h = FIX2INT(argv[4]);	
  widget->call_parent_draw_row(FIX2INT(argv[0]),x,y,w,h,FIX2INT(argv[5]));
  return Qnil;
};


extern "C" void
Init_flvlist(){
  rb_cFLVList = rb_define_class_under(rb_mFLTK, "VList", rb_cFLGroup);
  DEF_S_METHOD(VList,new,-1);
  DEF_INIT_METHOD(VList);
  DEF_WIDGET_METHODS(VList);
  DEF_METHOD(VList, global_style, 0);
  DEF_METHOD(VList, row_style, 0);
  DEF_METHOD(VList, get_style, -1);
  DEF_METHOD(VList, get_default_style, -1);
  DEF_METHOD(VList, row_height, -1);
  DEF_METHOD(VList, draw_row, -1);
  DEF_METHOD(VList, row_width, -1);
  DEF_METHOD(VList, feature, -1);
  DEF_METHOD(VList, feature_add, 1);
  DEF_METHOD(VList, feature_remove, 1);
  DEF_METHOD(VList, feature_test, 1);
  DEF_METHOD(VList, row, -1);
  DEF_METHOD(VList, rows, -1);
  DEF_METHOD(VList, has_scrollbar, -1);
  DEF_METHOD(VList, scrollbar, 0);
  DEF_METHOD(VList, hscrollbar, 0);
  DEF_METHOD(VList, max_clicks, -1);
  DEF_METHOD(VList, select_locked, -1);
  DEF_METHOD(VList, edit_when, -1);
  DEF_METHOD(VList, start_edit, 0);
  DEF_METHOD(VList, end_edit, 0);
  DEF_METHOD(VList, cancel_edit, 0);
  DEF_METHOD(VList, load_editor, -1);
  DEF_METHOD(VList, save_editor, -1);
  DEF_METHOD(VList, position_editor, -1);
  DEF_METHOD(VList, callback_when, -1);
  DEF_METHOD(VList, add_callback_when, 1);
  DEF_METHOD(VList, clear_callback_when, 1);
  DEF_METHOD(VList, callback_on, 1);
//  DEF_METHOD(VList, select, -1);
//  DEF_METHOD_EQ(Browser_, textcolor, -1);

  DEF_FLVE_CONST_NUM(ROW_CHANGED);
  DEF_FLVE_CONST_NUM(ROWS_CHANGED);
  DEF_FLVE_CONST_NUM(SELECTION_CHANGED);
  DEF_FLVE_CONST_NUM(COL_CHANGED);
  DEF_FLVE_CONST_NUM(COLS_CHANGED);
  DEF_FLVE_CONST_NUM(FEATURE_CHANGED);
  DEF_FLVE_CONST_NUM(CLICKED);
  DEF_FLVE_CONST_NUM(ENTER_PRESSED);
  DEF_FLVE_CONST_NUM(ROW_HEADER_CLICKED);
  DEF_FLVE_CONST_NUM(ROW_FOOTER_CLICKED);
  DEF_FLVE_CONST_NUM(COL_HEADER_CLICKED);
  DEF_FLVE_CONST_NUM(COL_FOOTER_CLICKED);
  DEF_FLVE_CONST_NUM(LABEL_CLICKED);
  DEF_FLVE_CONST_NUM(TITLE_CLICKED);
  DEF_FLVE_CONST_NUM(ALL_CLICKED);

  DEF_FLVECB_CONST_NUM(ROW_CHANGED);
  DEF_FLVECB_CONST_NUM(ROWS_CHANGED);
  DEF_FLVECB_CONST_NUM(SELECTION_CHANGED);
  DEF_FLVECB_CONST_NUM(COL_CHANGED);
  DEF_FLVECB_CONST_NUM(COLS_CHANGED);
  DEF_FLVECB_CONST_NUM(FEATURE_CHANGED);
  DEF_FLVECB_CONST_NUM(CLICKED);
  DEF_FLVECB_CONST_NUM(ENTER_PRESSED );
  DEF_FLVECB_CONST_NUM(ROW_HEADER_CLICKED);
  DEF_FLVECB_CONST_NUM(ROW_FOOTER_CLICKED);
  DEF_FLVECB_CONST_NUM(COL_HEADER_CLICKED);
  DEF_FLVECB_CONST_NUM(COL_FOOTER_CLICKED);
  DEF_FLVECB_CONST_NUM(LABEL_CLICKED);
  DEF_FLVECB_CONST_NUM(TITLE_CLICKED);
  DEF_FLVECB_CONST_NUM(ALL_CLICKED);

  DEF_FLVF_CONST_NUM(NONE);
  DEF_FLVF_CONST_NUM(ROW_HEADER);
  DEF_FLVF_CONST_NUM(ROW_FOOTER);
  DEF_FLVF_CONST_NUM(ROW_DIVIDER);
  DEF_FLVF_CONST_NUM(COL_HEADER);
  DEF_FLVF_CONST_NUM(COL_FOOTER);
  DEF_FLVF_CONST_NUM(COL_DIVIDER);
  DEF_FLVF_CONST_NUM(MULTI_SELECT);
  DEF_FLVF_CONST_NUM(ROW_SELECT);
  DEF_FLVF_CONST_NUM(PERSIST_SELECT);
  DEF_FLVF_CONST_NUM(FULL_RESIZE);
  DEF_FLVF_CONST_NUM(DIVIDERS);
  DEF_FLVF_CONST_NUM(HEADERS);
  DEF_FLVF_CONST_NUM(FOOTERS);
  DEF_FLVF_CONST_NUM(ROW_ENDS);
  DEF_FLVF_CONST_NUM(COL_ENDS);
  DEF_FLVF_CONST_NUM(ALL_ROW);
  DEF_FLVF_CONST_NUM(ALL_COL);

  DEF_FLVS_CONST_NUM(NONE);
  DEF_FLVS_CONST_NUM(HORIZONTAL);
  DEF_FLVS_CONST_NUM(VERTICAL);
  DEF_FLVS_CONST_NUM(BOTH);

  DEF_FLVS_CONST_NUM(AUTOMATIC);
  DEF_FLVS_CONST_NUM(ALWAYS);

  DEF_FLVS_CONST_NUM(HORIZONTAL_ALWAYS);
  DEF_FLVS_CONST_NUM(VERTICAL_ALWAYS);
//  DEF_FLVS_CONST_NUM(BOTH_ALWAYS);

  DEF_FLV_CONST_NUM(EDIT_ALWAYS);		
  DEF_FLV_CONST_NUM(EDIT_AUTOMATIC);    
  DEF_FLV_CONST_NUM(EDIT_MANUAL);		
}
