/* -*- C++ -*-
 * $Id: flvlist.h,v 1.2 2005/05/22 23:22:43 jeremy_henty Exp $
 */

#ifndef RBFLVLIST_H
#define RBFLVLIST_H

/*
 * select macro is defined in ruby-x.x.x/win32/win32.h.
 * This definition conflicts with Fl_Browser::select.
 *                                  Thanks Lorien Dunn
 */
#ifdef select
# undef select
#endif

#include <ruby.h>
#include <FL/Flv_List.H>
#include "../flgroup.h"
#include "flvstyle.h"
#include "../flvaluator.h"

class RBFLVList;

class RBFLVListInternalWidgetsMask
{
public:
    RBFLVList *vl;
    VALUE rb_scrollbar, rb_hscrollbar, rb_global_style, rb_row_style;
    RBFLVListInternalWidgetsMask() {vl = NULL;};
    ~RBFLVListInternalWidgetsMask();
    void init_RBFLVListInternalWidgetsMask(RBFLVList *p);
};

class RBFLVList : public Flv_List, public RBFLVListInternalWidgetsMask
{
public:
  RBFLVList(int x, int y, int w, int h, const char *label = 0):
    Flv_List(x,y,w,h,label),RBFLVListInternalWidgetsMask(){
	init_RBFLVListInternalWidgetsMask(this);
  };
  RBFL_CALLERS(Flv_List);


  virtual int row_height(int n) {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE ret = rb_funcall(obj, rb_intern("row_height"), 1, INT2FIX(n));
    return FIX2INT(ret);
  };

  int call_parent_row_height(int n) {
  int h;
    h = Flv_List::row_height(n);
    return h;
  };

  virtual int row_height(int n, int r) {
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE ret = rb_funcall(obj, rb_intern("row_height"), 2, INT2FIX(n), INT2FIX(r));
    return FIX2INT(ret);
  };

  int call_parent_row_height(int n, int r) {
    return Flv_List::row_height(n,r);
  };

  virtual void get_style(Flv_Style &s, int R, int C=0){	//	get trickle down style
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE style = Data_Wrap_Struct(rb_cFLVStyle,0,0,&s);
    rb_gc_register_address(&style);
    rb_funcall(obj, rb_intern("get_style"), 3, style, INT2FIX(R), INT2FIX(C));
    rb_gc_unregister_address(&style);
  }

  void call_parent_get_style(Flv_Style &s, int R, int C=0){	//	get trickle down style
    Flv_List::get_style(s,R,C);
  }

  virtual void save_editor(Fl_Widget *e, int R, int C=0){ //	Save editor contents
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE editor = fl_to_rb((Fl_Widget*)e);
    rb_funcall(obj, rb_intern("save_editor"), 3, editor, INT2FIX(R), INT2FIX(C));
  }

  virtual void call_parent_save_editor(Fl_Widget *e, int R, int C=0){ //	Save editor contents
    save_editor(e,R,C);
  }

  virtual void load_editor(Fl_Widget *e, int R, int C=0){ //	Load editor contents
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE editor = fl_to_rb((Fl_Widget*)e);
    rb_funcall(obj, rb_intern("load_editor"), 3, editor, INT2FIX(R), INT2FIX(C));
  }

  virtual void call_parent_load_editor(Fl_Widget *e, int R, int C=0){ //	Load editor contents
    Flv_List::load_editor(e,R,C);
  }

  virtual void position_editor(Fl_Widget *e, int x, int y, int w, int h, Flv_Style &s ){
    VALUE obj = fl_to_rb((Fl_Widget*)this);
    VALUE editor = fl_to_rb((Fl_Widget*)e);
    VALUE style = Data_Wrap_Struct(rb_cFLVStyle,0,0,&s);
    rb_gc_register_address(&style);
    rb_funcall(obj, rb_intern("position_editor"), 6, editor, INT2FIX(x), INT2FIX(y),
	INT2FIX(w), INT2FIX(h), style);
    rb_gc_unregister_address(&style);
  }

  virtual void call_parent_position_editor(Fl_Widget *e, int x, int y, int w, 
	int h, Flv_Style &s ){
	Flv_List::position_editor(e,x,y,w,h,s);
  }

  virtual void draw_row( int Offset, int &X, int &Y, int &W, int &H, int R ){
    VALUE rc, obj = fl_to_rb((Fl_Widget*)this);
    rc = rb_funcall(obj, rb_intern("draw_row"), 6, INT2FIX(Offset), INT2FIX(X), INT2FIX(Y), 
	INT2FIX(W), INT2FIX(H), INT2FIX(R));
  }

  void call_parent_draw_row( int Offset, int &X, int &Y, int &W, int &H, int R ){
    Flv_List::draw_row( Offset, X, Y, W, H, R );
  }
};


extern VALUE rb_cFLVList;
extern "C" void Init_flvlist(void);

#endif /* RBFLVLIST_H */
