/* -*- C++ -*-
 * $Id: flvtable.cc,v 1.6 2006/12/29 05:20:13 jeremy_henty Exp $
 */

#include "../fltk.h"
#include "flvlist.h"
#include "flvtable.h"
#include "flvstyle.h"
#include "../flenum.h"
#include "../flwidget.h"

#define DEF_FLV_CONST_NUM(c) \
  DEF_MODULE_CONST("V_"#c,INT2NUM(FLV_##c))

VALUE rb_cFLVTable;

WIDGET_FNS(VTable);

static FN_GETSET_INT(VTable, col);
static FN_GETSET_INT(VTable, cols);

static FN_GETSET_INT(VList, edit_when);
static FN_CALL(VList, start_edit);
static FN_CALL(VList, end_edit);
static FN_CALL(VList, cancel_edit);


static VALUE rb_FLVTable_initialize(int argc, VALUE argv[], VALUE self)
{
  return Qnil;
};


static VALUE rb_FLVTable_s_new(int argc, VALUE argv[], VALUE klass) 
{ 
  VALUE obj; 
  RBFLVTable *vl = NULL; 
  VALUE arg1,arg2,arg3,arg4,arg5;
  switch( rb_scan_args(argc, argv, "23", &arg1, &arg2, &arg3, &arg4, &arg5) ){
  case 2:
    vl = new RBFLVTable(0, 0, FIX2INT(arg1),FIX2INT(arg2));
    break;
  case 4:
    vl = new RBFLVTable(FIX2INT(arg1),FIX2INT(arg2),FIX2INT(arg3),FIX2INT(arg4));
    break;
  case 5:
    vl = new RBFLVTable(FIX2INT(arg1), FIX2INT(arg2), FIX2INT(arg3),FIX2INT(arg4),STR2CPTR(arg5));
    break;
  };
  obj = rb_fl_new(klass, vl); 
  rb_obj_call_init(obj, argc, argv); 
  return obj; 
};

static VALUE rb_FLVTable_global_style(VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  return widget->rb_global_style;
};

static VALUE rb_FLVTable_row_style(VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  return widget->rb_row_style;
};

static VALUE rb_FLVTable_scrollbar(VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  return widget->rb_scrollbar;
};

static VALUE rb_FLVTable_hscrollbar(VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  return widget->rb_hscrollbar;
};

static VALUE rb_FLVTable_col_style(VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  return widget->rb_col_style;
};

static VALUE rb_FLVTable_get_style(int argc, VALUE argv[], VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  VALUE arg1,arg2,arg3;

  Flv_Style *s; 
  Data_Get_Struct(argv[0],Flv_Style,s);

  switch( rb_scan_args(argc, argv, "12", &arg1, &arg2, &arg3) ){
  case 1:
    widget->call_parent_get_style(*s,0,0);
    break;
  case 2:
    widget->call_parent_get_style(*s,FIX2INT(arg2),0);
    break;
  case 3:
    widget->call_parent_get_style(*s,FIX2INT(arg2),FIX2INT(arg3));
    break;
  };
  return Qnil;
};

static VALUE rb_FLVTable_col_width(int argc, VALUE argv[], VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  VALUE arg1,arg2;
  int rc = 0;

  switch( rb_scan_args(argc, argv, "11", &arg1, &arg2) ){
  case 1:
    rc = widget->call_parent_col_width(FIX2INT(arg1));
    break;
  case 2:
    rc = widget->call_parent_col_width(FIX2INT(arg1),FIX2INT(arg2));
    break;
  };
  return INT2FIX(rc);
};

static VALUE rb_FLVTable_draw_row(int argc, VALUE argv[], VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  int x = FIX2INT(argv[1]),
      y = FIX2INT(argv[2]),
      w = FIX2INT(argv[3]),
      h = FIX2INT(argv[4]);	
  widget->call_parent_draw_row(FIX2INT(argv[0]),x,y,w,h,FIX2INT(argv[5]));
  return Qnil;
};

static VALUE rb_FLVTable_draw_cell(int argc, VALUE argv[], VALUE self)
{
  RBFLVTable *widget = (RBFLVTable*)rb_to_fl(self);
  int x = FIX2INT(argv[1]),
      y = FIX2INT(argv[2]),
      w = FIX2INT(argv[3]),
      h = FIX2INT(argv[4]);	
  widget->call_parent_draw_cell(FIX2INT(argv[0]),x,y,w,h,FIX2INT(argv[5]),FIX2INT(argv[6]));
  return Qnil;
};

extern "C" void
Init_flvtable(){
  rb_cFLVTable = rb_define_class_under(rb_mFLTK, "VTable", rb_cFLVList);
  DEF_S_METHOD(VTable,new,-1);
  DEF_INIT_METHOD(VTable);
  DEF_WIDGET_METHODS(VTable);
  DEF_METHOD(VTable, col_width, -1);
  DEF_METHOD(VTable, get_style, -1);
  DEF_METHOD(VTable, col_style, 0);
  DEF_METHOD(VTable, draw_row, -1);
  DEF_METHOD(VTable, draw_cell, -1);
  DEF_METHOD(VTable, cols, -1);
  DEF_METHOD(VTable, col, -1);
  DEF_METHOD(VTable, global_style, 0);
  DEF_METHOD(VTable, row_style, 0);
  DEF_METHOD(VTable, scrollbar, 0);
  DEF_METHOD(VTable, hscrollbar, 0);
  DEF_METHOD(VList, edit_when, -1);
  DEF_METHOD(VList, start_edit, 0);
  DEF_METHOD(VList, end_edit, 0);
  DEF_METHOD(VList, cancel_edit, 0);

  DEF_FLV_CONST_NUM(COL_HEADER);
  DEF_FLV_CONST_NUM(COL_FOOTER);
  DEF_FLV_CONST_NUM(MOVE_ON_ENTER_ROW_COL);
  DEF_FLV_CONST_NUM(MOVE_ON_ENTER_COL_ROW);
  DEF_FLV_CONST_NUM(MOVE_ON_ENTER_NEVER);
  DEF_FLV_CONST_NUM(BUTTON1);
  DEF_FLV_CONST_NUM(BUTTON2);
  DEF_FLV_CONST_NUM(BUTTON3);
}
